function classifyResult = challenge(recordName)
%
% Sample entry for the 2017 PhysioNet/CinC Challenge.
%
% INPUTS:
% recordName: string specifying the record name to process
%
% OUTPUTS:
% classifyResult: integer value where
%                     N = normal rhythm
%                     A = AF
%                     O = other rhythm
%                     ~ = noisy recording (poor signal quality)
%
% To run your entry on the entire training set in a format that is
% compatible with PhysioNet's scoring enviroment, run the script
% generateValidationSet.m
%
% The challenge function requires that you have downloaded the challenge
% data 'training_set' in a subdirectory of the current directory.
%    http://physionet.org/physiobank/database/challenge/2017/
%
% This dataset is used by the generateValidationSet.m script to create
% the annotations on your training set that will be used to verify that
% your entry works properly in the PhysioNet testing environment.
%
%
% Version 1.0
%
%
% Written by: Chengyu Liu and Qiao Li January 20 2017
%             chengyu.liu@emory.edu  qiao.li@emory.edu
%
% Last modified by:
%
%

%% Add Paths
addpath(genpath('.'));

%% Extract Ensemble Features and 'normalized' length-40 sorted RR intervals
[features,interpolated_RR_intervals] = extract_features(recordName,0);

%% Load dictionary and learn coefficients
load('SC_DICT.mat');
opts.parallel = 0;
opts.minibatch_size = 1;
coefs = learn_coefficients(dict, interpolated_RR_intervals', opts)';

%% Classify using SVM
addpath('libsvm');
load('SVM_SC.mat');
coefs = (coefs - avg_SC) ./ sd_SC;
[~,~,SC_scores] = svmpredict(0, coefs, svmModel_SC, '-q -b 1');

load('SVM.mat');
features = [features, SC_scores];
features = (features - avg) ./ sd;
[predLabel,~,~] = svmpredict(0, features, svmModel, '-q -b 1');

switch predLabel
    case 1
        classifyResult = 'N';
    case 2
        classifyResult = 'A';
    case 3
        classifyResult = 'O';
    otherwise
        classifyResult = '~';
end
