function [ecgDel, waveSign, badSegs] = BurakECGDel(curWaveform, fs, segmentTime, ecgFilt, isFigure)
% From a given ECG signal, returns the R peak locations.
%                       Outputs:
%
% ecgDel: Indices of the fiducial point locations in the curWaveform signal.
% ecgDel is a structure containing rPeaks, qPoints, sPoints, pPoints,
% tPoints
%
% badSegs: 2-dimensional array that returns the segment start and end
% indices for segments that can't be detected any R-peak. If all segments
% are OK returns an empty vector. Each row is a segment. First column is
% the segment start index and second column is the segment end index.
%
%
% sign: 1 if the polarity of R peak's are positive, -1 if otherwise.
%
%                       Inputs:
%
% curWaveform: ECG signal
%
% fs: sampling frequency of the ECG signal in samples/second.
%
% segmentTime: ECG signal is segmented and segmentTime determines the
% length of each segment in SECONDS.
%
% ecgFilt: ecg filter to be used in filtering the signal
%
% isFigure: plots figures if isFigure is true, otherwise it doesn't plot
% anything
%

%% Notes about To-Do and change:
% -Filter should be a part of this code, not sth that a user supplies.
% -To have the filter part of the r-peak detection code, adaptive
% filtering may be required.
fs = round(fs);
curWaveform = filtfilt(ecgFilt, curWaveform);
segmentLen = segmentTime*fs;
sigLen = length(curWaveform);
rPeakInds = zeros(round(sigLen/fs*4),1);
beatOverlap = 2;
badSegs = [];
%% First segment (loop invariables)
segStart = 1;
curRPeakInd = 1;
if (segmentLen < sigLen)
    segEnd = segmentLen;
else
    segEnd = sigLen;
end
isOverlapped = false;
overlappedRPeaks = 0;
%% Rest of the segments
while (segStart < sigLen)
    seg = curWaveform(segStart:segEnd);
    filtSeg = filtfilt(ecgFilt, seg);
    [phaseSignal, ampSignal] = BurakPTransform(filtSeg, 0.001);
    [rPeaks] = BurakSegtoRPeaks(phaseSignal, ampSignal, filtSeg, fs, false);
    % Check if rPeaks are valid. (check if current segment is bad)
    if (rPeaks == 0)
        isCurSegOK = false;
        badSegs = [badSegs; segStart, segEnd];
    else
        isCurSegOK = true;
    end
    rPeaks = rPeaks + segStart - 1; %Convert to indices for the whole signal.
    
    % Check if last beatOverlap peaks of the previous segment and first beatOverlap peaks of the current segment overlap.
    if (isCurSegOK)
        if (isOverlapped)
            prevRPeaks = rPeakInds(overlappedRPeaks);
            if (length(rPeaks) <= length(overlappedRPeaks)) %This part is complete garbage. Just my code not to fail.
                isMisMatch = false;
                rPeaks = rPeakInds(overlappedRPeaks);
            else
                isMisMatch = any(abs(prevRPeaks-rPeaks(1:length(overlappedRPeaks))) > 3);
            end
            if (isMisMatch)
                prevRPeaks = rPeakInds(rPeakInds>prevSegStart & rPeakInds<prevSegEnd);
                %                 fprintf('last beatOverlap peaks of the previous segment and first beatOverlap peaks of the current segment dont overlap\n')
                if (isFigure) % Plot both of the segments
                    figure(fHandle(1))
                    subplot(2,1,1)
                    title('Previous Segment')
                    prevSeg = curWaveform(prevSegStart:prevSegEnd);
                    plot(prevSeg)
                    prevRPeaks = prevRPeaks - prevSegStart + 1;
                    hold on
                    plot(prevRPeaks, prevSeg(prevRPeaks), 'ro')
                    hold off
                    subplot(2,1,2)
                    title('Current Segment')
                    plot(seg)
                    curRPeaks = rPeaks - segStart + 1;
                    hold on
                    plot(curRPeaks, seg(curRPeaks), 'ro')
                    hold off
                end
                %Decide which segment to accept.
                %Current rule: Choose the R-peaks in the segment in which
                %RR interval is closer to the mean RR interval in the
                %segment.
                meanPrevRPeaks = mean(diff(prevRPeaks));
                meanCurRPeaks = mean(diff(rPeaks));
                prevRRInt = prevRPeaks(end)-prevRPeaks(end-1);
                curRRInt = rPeaks(2)-rPeaks(1);
                curBeatOverlap = length(overlappedRPeaks);
                if (abs(curRRInt-meanCurRPeaks)<=abs(prevRRInt-meanPrevRPeaks))
                    rPeakInds(curRPeakInd-curBeatOverlap:curRPeakInd+length(rPeaks)-1-curBeatOverlap) = rPeaks;
                else
                    if (abs(rPeakInds(curRPeakInd-1) - rPeaks(curBeatOverlap+1)) > 4)
                        rPeakInds(curRPeakInd:curRPeakInd+length(rPeaks)-1-curBeatOverlap) = rPeaks(curBeatOverlap+1:end);
                    else
                        rPeakInds(curRPeakInd:curRPeakInd+length(rPeaks)-2-curBeatOverlap) = rPeaks(curBeatOverlap+2:end);
                        rPeaks(curBeatOverlap+1) = [];
                    end
                    
                end
                if (isFigure) % Plot both of the segments
                    figure(fHandle(1))
                    subplot(2,1,2)
                    title('Current Segment')
                    plot(seg)
                    curRPeaks = rPeaks - segStart + 1;
                    curRPeaks(1:curBeatOverlap) = rPeakInds(curRPeakInd-curBeatOverlap:curRPeakInd-1);
                    curRPeaks(1:curBeatOverlap) = curRPeaks(1:curBeatOverlap) - segStart + 1;
                    hold on
                    plot(curRPeaks, seg(curRPeaks), 'ro')
                    hold off
                end
                curRPeakInd = curRPeakInd+length(rPeaks)-curBeatOverlap;
                
            else
                curBeatOverlap = length(overlappedRPeaks);
                rPeakInds(curRPeakInd-curBeatOverlap:curRPeakInd+length(rPeaks)-1-curBeatOverlap) = rPeaks;
                curRPeakInd = curRPeakInd+length(rPeaks)-curBeatOverlap;
            end
            
        else
            rPeakInds(curRPeakInd:curRPeakInd+length(rPeaks)-1) = rPeaks;
            curRPeakInd = curRPeakInd+length(rPeaks);
        end
        %Advance to the next segment
        
        if (any(diff(rPeakInds(1:curRPeakInd-1)) < 0))
            fprintf('problem\n')
        end
        prevSegStart = segStart;
        prevSegEnd = segEnd;
        segStart = rPeakInds(curRPeakInd - beatOverlap) - round(0.05*fs); %50ms before the R-Peak
        overlappedRPeaks = curRPeakInd-beatOverlap:curRPeakInd-1;
        segEnd = segStart + segmentLen - 1;
        
    else
        prevSegStart = segStart;
        prevSegEnd = segEnd;
        % Design choice: if the current segment is bad, for the next segment, there is no overlap with the current segment
        segStart = segEnd + 1;
        segEnd = segStart + segmentLen - 1;
        overlappedRPeaks = 0;
    end
    if (segEnd >= sigLen)
        if (prevSegEnd == sigLen) % if previous segment was the last segment, then exit the loop.
            segStart = sigLen;
        else
            lastSegmentDur = (sigLen-prevSegEnd)/fs;
            if (lastSegmentDur < 2) % if the last segment is less than 2 seconds, exit the loop
                segStart = sigLen;
            elseif (lastSegmentDur < 4) % if the last segment is less than 4 seconds but greater than 2 seconds
                segStart = sigLen-segmentLen+1; % Make the last 6 seconds as
                if (prevSegEnd - segStart > 0)
                    %This part could be written more efficiently
                    overlappedRPeaks = find(rPeakInds > segStart);
                else
                    overlappedRPeaks = 0;
                end
            end
            segEnd = sigLen;
        end
    end
    isOverlapped = prevSegEnd - segStart > 0;
end

rPeakInds = rPeakInds(1:curRPeakInd-1);
if (isempty(rPeakInds))
    ecgDel = 0;
    waveSign = 0;
    return
end
%% Phasor Transform Overall Signal
[phaseSignal, ampSignal] = BurakPTransform(curWaveform, 0.001);
rPeakInds = BurakCorrectRPks2(rPeakInds, phaseSignal, ampSignal, curWaveform, fs);
while (any(diff(rPeakInds) <= 0))
    rPeakInds(find(diff(rPeakInds) < 0) + 1) = [];
    rPeakInds(diff(rPeakInds) == 0) = [];
end
%% Detect overall sign of the waveform and individuals polarities.
if (sum(sign(curWaveform(rPeakInds))) > 0)
    waveSign = 1;
else
    waveSign = -1;
end
pols = waveSign.*sign(curWaveform(rPeakInds));
% if (isFigure) %Plot the signal
%     figure(fHandle(2))
%     plot(sign.*curWaveform)
%     title('R-Peak Results')
%     hold on
%     plot(rPeakInds, sign.*curWaveform(rPeakInds), 'ro')
%     hold off
% end



%% QRS Delineation

if (waveSign == -1)
    newSignal = waveSign.*curWaveform;
else
    newSignal = curWaveform;
end

QInds = zeros(length(rPeakInds),1);
SInds = zeros(length(rPeakInds),2);
% first column max and second column offset.
PInds = zeros(length(rPeakInds),3);
%First column is P peak, second one is onset and third one is the offset index
TInds = zeros(length(rPeakInds),3);
%First column is T peak, second one is onset and third one is the offset index
for i = 1:length(rPeakInds)
    if (i == 1)
        isLeftOK = false;
        isRightOK = true;
    elseif (i == length(rPeakInds))
        isLeftOK = true;
        isRightOK = false;
    else
        isLeftOK = true;
        isRightOK = true;
    end
    if (isLeftOK && isRightOK)
        sigToCons = newSignal(rPeakInds(i-1):rPeakInds(i+1));
        rPeakPos = rPeakInds(i)-rPeakInds(i-1)+1;
    elseif (isLeftOK)
        sigToCons = newSignal(rPeakInds(i-1):rPeakInds(i));
        rPeakPos = length(sigToCons);
    elseif (isRightOK)
        sigToCons = newSignal(rPeakInds(i):rPeakInds(i+1));
        rPeakPos = 1;
    end
    windowLen = round(35*1e-3*fs); %35ms window
    if (isLeftOK)
        isLeftLong = rPeakInds(i)-rPeakInds(i-1) > (segmentTime-1)*fs;
    else
        isLeftLong = false;
    end
    if (isRightOK)
        isRightLong = rPeakInds(i+1)-rPeakInds(i) > (segmentTime-1)*fs;
    else
        isRightLong = false;
    end
    medianRRInterval = median(diff(rPeakInds));
    
    % Find Q
    if (isLeftOK)
        [gammaNeg, isQValid] = getGamma(phaseSignal(rPeakInds(i-1):rPeakInds(i)), rPeakPos, 'Q');
        if (isQValid)
            winStart = gammaNeg-windowLen;
            winEnd = gammaNeg;
            try
            posSig = sigToCons(winStart:winEnd) - median(sigToCons(winStart:winEnd));
            catch
                
            end
            [newPhaseSig, newAmpSig] = BurakPTransform(posSig, 0.005);
            if (isFigure && ~isempty(f3))
                figure(f3)
                subplot(3,1,1)
                plot(posSig)
                subplot(3,1,2)
                plot(newPhaseSig)
                subplot(3,1,3)
                plot(newAmpSig)
            end
            %         [posPhases, Qind] = findpeaks(-1.*newPhaseSig);
            [~, locMins] = findpeaks(-1.*newPhaseSig);
            if (isempty(locMins))
                qPIndex = winStart;
            else
                qPIndex = winStart + locMins(1) - 1;
            end
            QInds(i) = qPIndex + rPeakInds(i-1) - 1;
            
        else
            QInds(i) = NaN;
        end
    else
        QInds(i) = NaN;
    end
    
    
    % Find S
    if (isRightOK)
        [gammaPos, isSValid] = getGamma(phaseSignal(rPeakInds(i):rPeakInds(i+1)), rPeakPos, 'S');
        if (isSValid)
%             gammaPos = gammaPos + rPeakInds(i) - 1;
            winStart = gammaPos;
            winEnd = gammaPos + windowLen;
            posSig = sigToCons(winStart:winEnd) - median(sigToCons(winStart:winEnd));
            [newPhaseSig, newAmpSig] = BurakPTransform(posSig, 0.005);
            [~, locMins] = findpeaks(-1.*newPhaseSig);
            if (isempty(locMins))
                sPIndex = winEnd;
            else
                sPIndex = winStart + locMins(end) - 1;
            end
            SInds(i,2) = sPIndex + rPeakInds(i) - 1;
            [~, maxInd] = max(abs(sigToCons(winStart:winEnd)));
            sPIndex2 = maxInd + winStart - 1;
            SInds(i,1) = maxInd + winStart + rPeakInds(i) - 2;
            %             candidatePoints = newPhaseSig > max(newPhaseSig)*0.5;
            %             if nnz(candidatePoints) == 1
            %                 sPoint = find(candidatePoints);
            %             else
            %                 [~, maxInd] = max(newAmpSig(candidatePoints));
            %                 sPoint = find(candidatePoints);
            %                 sPoint = sPoint(maxInd);
            %             end
            %             sPIndex = winStart + sPoint - 1;
        else
            SInds(i,:) = NaN;
        end
    else
        SInds(i, :) = NaN;
    end
    
    
    
    
    % Find P
    if (isLeftOK && isQValid)
        isPDetected = false;
        winDec = round(fs*30e-3);
        if (~isLeftLong)
            windowLen = round(0.25.*(rPeakInds(i)-rPeakInds(i-1))) + winDec;
        else
            windowLen = round(0.25.*medianRRInterval) + winDec;
        end
        try
        windowEnd = qPIndex - round(fs*20e-3);
        catch
            
        end
        pLoc = NaN;
        while (~isPDetected && (windowLen/fs)>35e-3) %Possible optimization here. No Need to recalculate phasor transform in each iteration of the loop.
            windowLen = windowLen - winDec;
            windowStart = windowEnd-windowLen;
            try
            newSig = sigToCons(windowStart:windowEnd)-median(sigToCons(windowStart:windowEnd));
            catch
                
            end
            %             isoLevel = median(sigToCons(windowStart:windowEnd));
            %             stdLevel = std(sigToCons(windowStart:windowEnd));
            [phaseSig, ampSig] = BurakPTransform(newSig, 0.005);
            if (isFigure && ~isempty(f4))
                figure(f4)
                subplot(3,1,1)
                plot(sigToCons(windowStart:windowEnd))
                subplot(3,1,2)
                plot(newSig)
                subplot(3,1,3)
                plot(phaseSig)
            end
            [maxPhases, maxInd] = findpeaks(phaseSig, 'MinPeakProminence', max(phaseSig)/2);
            if (~isempty(maxInd))
                [~, indIndMax] = max(maxPhases);
                pLoc = maxInd(indIndMax) + windowStart - 1;
                isPDetected = true;
            else
                [maxPhases, maxInd] = findpeaks(phaseSig);
                if (~isempty(maxInd))
                    pLoc = maxInd(1) + windowStart - 1;
                    isPDetected = true;
                end
            end
            %             [~, maxInd] = max(phaseSig);
            %             if (maxInd == length(phaseSig))
            %                 pLoc = NaN;
            %                 isPDetected = true;
            %             elseif (maxInd ~= 1)
            %                 if (ampSig(maxInd) > ampSig(maxInd-1) && ampSig(maxInd) > ampSig(maxInd+1))
            %                     isPDetected = true;
            %                     pLoc = maxInd + windowStart - 1;
            %                 end
            %             end
        end
        if (~isnan(pLoc))
            if (true)%(sigToCons(pLoc) > 0*sigToCons(rPeakPos))
                [~, minInds] = findpeaks(-1.*phaseSig, 'MinPeakProminence', max(phaseSig)/3);
                pOnInd = find((minInds + windowStart - 1 < pLoc), 1, 'last');
                if (isempty(pOnInd))
                    pOnset = windowStart;
                else
                    pOnset = minInds(pOnInd) + windowStart - 1;
                end
                pOffInd = find((minInds + windowStart - 1) > pLoc, 1, 'first');
                if (isempty(pOffInd))
                    pOffset = windowEnd;
                else
                    pOffset = minInds(pOffInd) + windowStart - 1;
                end
                PInds(i, 1) = pLoc + rPeakInds(i-1) - 1;
                PInds(i, 2) = pOnset + rPeakInds(i-1) - 1;
                PInds(i, 3) = pOffset + rPeakInds(i-1) - 1;
            end
        else
            PInds(i,:) = NaN;
        end
    else
        PInds(i,:) = NaN;
    end
    % Find T
    if(isRightOK && isSValid)
        isTDetected = false;
        winInc = round(fs*30e-3);
        if (~isRightLong)
            windowLen = round(0.4.*(rPeakInds(i+1)-rPeakInds(i)));
            lastCons = round(0.6.*(rPeakInds(i+1)-rPeakInds(i)));
        else
            windowLen = round(0.4.*medianRRInterval);
            lastCons = round(0.6.*medianRRInterval);
        end
        windowStart = sPIndex + round(fs*50e-3);
        windowEnd = windowStart + windowLen;
        tLoc = NaN;
        while (~isTDetected && (windowLen)<lastCons && windowEnd < length(sigToCons)) %Possible optimization here. No Need to recalculate phasor transform in each iteration of the loop.
            newSig = sigToCons(windowStart:windowEnd)-median(sigToCons(windowStart:windowEnd));
%             isoLevel = median(sigToCons(windowStart:windowEnd));
%             stdLevel = std(sigToCons(windowStart:windowEnd));
            [phaseSig, ampSig] = BurakPTransform(newSig, 0.1);
            [maxPhases, maxInd] = findpeaks(phaseSig, 'MinPeakProminence', max(phaseSig)/2);
            if (~isempty(maxInd))
                [~, indIndMax] = max(maxPhases);
                tLoc = maxInd(indIndMax) + windowStart - 1;
                isTDetected = true;
                %             else
                %                 fprintf('problem\n')
            end
            windowLen = windowLen + winInc;
            windowEnd = windowStart+windowLen;
            %             [~, maxInd] = max(phaseSig);
            %             if (maxInd ~= length(phaseSig) && maxInd ~= 1)
            %                 if (ampSig(maxInd) > ampSig(maxInd-1) && ampSig(maxInd) > ampSig(maxInd+1))
            %                     isTDetected = true;
            %                     tLoc = maxInd + windowStart - 1;
            %                 end
            %             end
        end
        
        if (~isnan(tLoc))
            if (true)%(sigToCons(pLoc) > 0*sigToCons(rPeakPos))
                [~, minInds] = findpeaks(-1.*phaseSig, 'MinPeakProminence', max(phaseSig)/20);%, 'MinPeakProminence', max(phaseSig)/4);
                tOnInd = find((minInds + windowStart - 1 < tLoc), 1, 'last');
                if (isempty(tOnInd))
                    tOnset = windowStart;
                else
                    tOnset = minInds(tOnInd) + windowStart - 1;
                end
                tOffInd = find((minInds + windowStart - 1) > tLoc, 1, 'first');
                if (isempty(tOffInd))
                    tOffset = windowEnd;
                else
                    tOffset = minInds(tOffInd) + windowStart - 1;
                end
            end
            if (isLeftOK)
                TInds(i, 1) = tLoc + rPeakInds(i-1) - 1;
                TInds(i, 2) = tOnset + rPeakInds(i-1) - 1;
                TInds(i, 3) = tOffset + rPeakInds(i-1) - 1;
            else
                TInds(i, 1) = tLoc + rPeakInds(i) - 1;
                TInds(i, 2) = tOnset + rPeakInds(i) - 1;
                TInds(i, 3) = tOffset + rPeakInds(i) - 1;
            end
        else
            TInds(i,:) = NaN;
        end
        
    else
        TInds(i,:) = NaN;
        
    end
    
    if (isFigure && ishandle(f2) && ~isempty(f2) && isSValid && isQValid)
        figure(f2)
        vecToPlot = curWaveform(rPeakInds(i-1):rPeakInds(i+1));
        plot(vecToPlot)
        hold on
        plot(sPIndex, vecToPlot(sPIndex), 'ko', 'DisplayName', 's')
        plot(sPIndex2, vecToPlot(sPIndex2), 'yo', 'DisplayName', 's2')
        %             plot(winStart, vecToPlot(winStart), 'ro')
        %             plot(winEnd, vecToPlot(winEnd), 'ro')
        winEnd = qPIndex;
        windowLen = round(0.25.*(rPeakInds(i)-rPeakInds(i-1)));
        winStart = winEnd-windowLen;
        %             sigVal = median(vecToPlot(winStart:winEnd));
        %             asdasd = round((winStart+winEnd)/2);
        plot(qPIndex, vecToPlot(qPIndex), 'go', 'DisplayName', 'q')
        %             plot(asdasd, sigVal, 'ro', 'DisplayName', 'Median Value')
        %             pInd = pLoc - (rPeakInds(i)-rPeakLoc)
        if (~isnan(pLoc))
            plot(pLoc, vecToPlot(pLoc), 'ro', 'DisplayName', 'P')
            plot(pOffset, vecToPlot(pOffset), 'r.','DisplayName', 'Poffset')
            plot(pOnset, vecToPlot(pOnset), 'r.','DisplayName', 'Ponnset')
        end
        if (~isnan(tLoc))
            plot(tLoc, vecToPlot(tLoc), 'co', 'DisplayName', 'T')
            plot(tOffset, vecToPlot(tOffset), 'c.', 'DisplayName', 'Toffset')
            plot(tOnset, vecToPlot(tOnset), 'c.', 'DisplayName', 'TOnset')
        end
        hold off
        legend('show')
    end
    
    
    
end %end if




%     windowLen = round(150e-3*fs);
%     windowEnd = rPeakInds(i)-1;
%     windowStart = windowEnd-windowLen;
%     derSignal = diff(signal(windowStart:windowEnd));
%     sigToPlot = signal(windowStart:windowEnd);
%     indNeg = find(sign(sigToPlot) == -1, 1, 'last');
%     indNegDer = find(sign(derSignal) == -1, 1, 'last');
%     if (isFigure && ishandle(f2) && ~isempty(f2))
%        figure(f2)
%        subplot(2,1,1)
%        plot(sigToPlot)
%        hold on
%        plot(indNeg, sigToPlot(indNeg), 'ro')
%        hold off
%        subplot(2,1,2)
%        plot(derSignal)
%        hold on
%        plot(indNegDer, derSignal(indNegDer), 'ro')
%        hold off
%     end

ecgDel.rPeaks = rPeakInds;
ecgDel.qPoints = QInds;
ecgDel.sPoints = SInds;
ecgDel.pPoints = PInds;
ecgDel.tPoints = TInds;
ecgDel.rPols = pols;


    function [gammaToReturn, isValid] = getGamma(phaseSeg, rPeakLoc, QorS)
        timeWindowToLookFor = 75e-3; %75 ms;
        isValid = true;
        increment = 0.005;
        basePercentage = 0.965;
        
        if (QorS == 'Q' || QorS == 'q')
            gammaToReturn = 1;
            while ((rPeakLoc-gammaToReturn)/fs > timeWindowToLookFor)
                basePercentage = basePercentage + increment;
                thresholdForGamma = pi/2*basePercentage;
                posGammaLocs = phaseSeg <= thresholdForGamma;
                if (all(posGammaLocs))
                    isValid = false;
                    break;
                end
                gammaToReturn = find(posGammaLocs, 1, 'last');
                if (isempty(gammaToReturn))
                   gammaToReturn = 1; 
                end
            end
        else
            gammaToReturn = Inf;
            while (gammaToReturn/fs > timeWindowToLookFor)
                basePercentage = basePercentage + increment;
                thresholdForGamma = pi/2*basePercentage;
                posGammaLocs = phaseSeg <= thresholdForGamma;
                if (all(posGammaLocs))
                    isValid = false;
                    break;
                end
                gammaToReturn = find(posGammaLocs, 1);
                if (isempty(gammaToReturn))
                   gammaToReturn = Inf; 
                end
            end
        end
        
    end


end

