function [rPeakInds, sign, badSegs] = BurakRPeakDet(curWaveform, fs, segmentTime, ecgFilt, isFigure)
% From a given ECG signal, returns the R peak locations.
%                       Outputs:
%
% rPeakInds: Indices of the R peak locations in the curWaveform signal.
%
% badSegs: 2-dimensional array that returns the segment start and end 
% indices for segments that can't be detected any R-peak. If all segments
% are OK returns an empty vector. Each row is a segment. First column is
% the segment start index and second column is the segment end index.
% 
%
% sign: 1 if the polarity of R peak's are positive, -1 if otherwise.
%
%                       Inputs:
%
% curWaveform: ECG signal
%
% fs: sampling frequency of the ECG signal in samples/second.
%
% segmentTime: ECG signal is segmented and segmentTime determines the
% length of each segment in SECONDS.
%
% ecgFilt: ecg filter to be used in filtering the signal
%
% fHandle: vector of figure handles to plot debugging figures. No figures
% if it is empty.
%

%% Notes about To-Do and change:
% -Filter should be a part of this code, not sth that a user supplies.
% -To have the filter part of the r-peak detection code, adaptive
% filtering may be required.
fs = round(fs);
curWaveform = filtfilt(ecgFilt, curWaveform);
segmentLen = segmentTime*fs;
sigLen = length(curWaveform);
rPeakInds = zeros(round(sigLen/fs*4),1);
beatOverlap = 2;
badSegs = [];
%% First segment (loop invariables)
segStart = 1;
curRPeakInd = 1;
if (segmentLen < sigLen)
    segEnd = segmentLen;
else
    segEnd = sigLen;
end
isOverlapped = false;
overlappedRPeaks = 0;
%% Rest of the segments
while (segStart < sigLen) 
    seg = curWaveform(segStart:segEnd);
    filtSeg = filtfilt(ecgFilt, seg);
    [phaseSignal, ampSignal] = BurakPTransform(filtSeg, 0.001);
    [rPeaks] = BurakSegtoRPeaks(phaseSignal, ampSignal, filtSeg, fs, false);
    % Check if rPeaks are valid. (check if current segment is bad)
    if (rPeaks == 0)
        isCurSegOK = false;
        badSegs = [badSegs; segStart, segEnd];
    else
        isCurSegOK = true;
    end
    rPeaks = rPeaks + segStart - 1; %Convert to indices for the whole signal.
    
    % Check if last beatOverlap peaks of the previous segment and first beatOverlap peaks of the current segment overlap.
    if (isCurSegOK)
        if (isOverlapped)
            prevRPeaks = rPeakInds(overlappedRPeaks);
            if (length(rPeaks) <= length(overlappedRPeaks)) %This part is complete garbage. Just my code not to fail.
                isMisMatch = false;
                rPeaks = rPeakInds(overlappedRPeaks);
            else
                isMisMatch = any(abs(prevRPeaks-rPeaks(1:length(overlappedRPeaks))) > 3);
            end
            if (isMisMatch)
                prevRPeaks = rPeakInds(rPeakInds>prevSegStart & rPeakInds<prevSegEnd);
                fprintf('last beatOverlap peaks of the previous segment and first beatOverlap peaks of the current segment dont overlap\n')
                if (isFigure) % Plot both of the segments
                    figure(fHandle(1))
                    subplot(2,1,1)
                    title('Previous Segment')
                    prevSeg = curWaveform(prevSegStart:prevSegEnd);
                    plot(prevSeg)
                    prevRPeaks = prevRPeaks - prevSegStart + 1;
                    hold on
                    plot(prevRPeaks, prevSeg(prevRPeaks), 'ro')
                    hold off
                    subplot(2,1,2)
                    title('Current Segment')
                    plot(seg)
                    curRPeaks = rPeaks - segStart + 1;
                    hold on
                    plot(curRPeaks, seg(curRPeaks), 'ro')
                    hold off
                end
                %Decide which segment to accept.
                %Current rule: Choose the R-peaks in the segment in which
                %RR interval is closer to the mean RR interval in the
                %segment.
                meanPrevRPeaks = mean(diff(prevRPeaks));
                meanCurRPeaks = mean(diff(rPeaks));
                prevRRInt = prevRPeaks(end)-prevRPeaks(end-1);
                curRRInt = rPeaks(2)-rPeaks(1);
                curBeatOverlap = length(overlappedRPeaks);
                if (abs(curRRInt-meanCurRPeaks)<=abs(prevRRInt-meanPrevRPeaks))
                    rPeakInds(curRPeakInd-curBeatOverlap:curRPeakInd+length(rPeaks)-1-curBeatOverlap) = rPeaks;
                else
                    if (abs(rPeakInds(curRPeakInd-1) - rPeaks(curBeatOverlap+1)) > 4)
                        rPeakInds(curRPeakInd:curRPeakInd+length(rPeaks)-1-curBeatOverlap) = rPeaks(curBeatOverlap+1:end);
                    else
                        rPeakInds(curRPeakInd:curRPeakInd+length(rPeaks)-2-curBeatOverlap) = rPeaks(curBeatOverlap+2:end);
                        rPeaks(curBeatOverlap+1) = [];
                    end

                end
                if (isFigure) % Plot both of the segments
                    figure(fHandle(1))
                    subplot(2,1,2)
                    title('Current Segment')
                    plot(seg)
                    curRPeaks = rPeaks - segStart + 1;
                    curRPeaks(1:curBeatOverlap) = rPeakInds(curRPeakInd-curBeatOverlap:curRPeakInd-1);
                    curRPeaks(1:curBeatOverlap) = curRPeaks(1:curBeatOverlap) - segStart + 1;
                    hold on
                    plot(curRPeaks, seg(curRPeaks), 'ro')
                    hold off
                end
                curRPeakInd = curRPeakInd+length(rPeaks)-curBeatOverlap;
                
            else
                curBeatOverlap = length(overlappedRPeaks);
                rPeakInds(curRPeakInd-curBeatOverlap:curRPeakInd+length(rPeaks)-1-curBeatOverlap) = rPeaks;
                curRPeakInd = curRPeakInd+length(rPeaks)-curBeatOverlap;
            end
            
        else
            rPeakInds(curRPeakInd:curRPeakInd+length(rPeaks)-1) = rPeaks;
            curRPeakInd = curRPeakInd+length(rPeaks);
        end
        %Advance to the next segment
        
        if (any(diff(rPeakInds(1:curRPeakInd-1)) < 0))
            fprintf('problem\n')
        end
        prevSegStart = segStart;
        prevSegEnd = segEnd;
        segStart = rPeakInds(curRPeakInd - beatOverlap) - round(0.05*fs); %50ms before the R-Peak
        overlappedRPeaks = curRPeakInd-beatOverlap:curRPeakInd-1;
        segEnd = segStart + segmentLen - 1;
        
    else
        prevSegStart = segStart;
        prevSegEnd = segEnd;
        % Design choice: if the current segment is bad, for the next segment, there is no overlap with the current segment
        segStart = segEnd + 1;
        segEnd = segStart + segmentLen - 1;
        overlappedRPeaks = 0;
    end
    if (segEnd >= sigLen)
        if (prevSegEnd == sigLen) % if previous segment was the last segment, then exit the loop.
            segStart = sigLen;
        else
            lastSegmentDur = (sigLen-prevSegEnd)/fs;
            if (lastSegmentDur < 2) % if the last segment is less than 2 seconds, exit the loop
                segStart = sigLen;
            elseif (lastSegmentDur < 4) % if the last segment is less than 4 seconds but greater than 2 seconds
                segStart = sigLen-segmentLen+1; % Make the last 6 seconds as
                if (prevSegEnd - segStart > 0)
                    %This part could be written more efficiently
                    overlappedRPeaks = find(rPeakInds > segStart);
                else
                    overlappedRPeaks = 0;
                end
            end
            segEnd = sigLen;
        end
    end
    isOverlapped = prevSegEnd - segStart > 0;
end

rPeakInds = rPeakInds(1:curRPeakInd-1);
if (mean(curWaveform) < mean(curWaveform(rPeakInds)))
    sign = 1;
else
    sign = -1;
end
if (isFigure) %Plot the signal
    figure(fHandle(2))
    plot(sign.*curWaveform)
    title('R-Peak Results')
    hold on
    plot(rPeakInds, sign.*curWaveform(rPeakInds), 'ro')
    hold off
end