%% Extract Features

function [features,interpolated_RR_intervals] = extract_features(recordName,show_plot)

    %% Parse Arguments
    if nargin == 1
        show_plot = 0;
    end
    
    interp_length = 50;

    %% Read file
    [tm,ecg,fs,~]=rdmat(recordName);
    
    %% Estimate R Times, Peak values, and RR-intervals using code provided by Challenge

    [QRS,~,~] = qrs_detect2(ecg',0.25,0.6,fs);
    
    R_times = QRS/fs;
    R_values = ecg(QRS);
    RR_intervals = diff(QRS)/fs;
    
    %% Estimate R Times, Peak values, and RR-intervals using Burak's code
    load Default_BP_Filter;
    addpath('Burak_R_Peak_Detector');
    [RPeak,~,badSegs] = BurakRPeakDet(ecg',fs,6,ECGBpFilt,[]);
    numBadSegs = size(badSegs,1);
    
    Burak_R_times = RPeak/fs;
    Burak_R_values = ecg(RPeak);
    Burak_RR_intervals = diff(RPeak)/fs;
    
    % Remove RR intervals that span noisy segments (greater than 3 seconds)
    Burak_RR_intervals = Burak_RR_intervals(Burak_RR_intervals < 3);
    
    
    %% (Optional) Plot the normalized sequence and the exctracted R peaks
    if show_plot
        figure(1);
        plot(tm,ecg); hold on;
        scatter(R_times,R_values,'rx');
        scatter(Burak_R_times,Burak_R_values,'bo');
        hold off;
    end
    
    %% Take care of special cases
    % 2 R-values = one RR-interval (not enough for interpolation)
    if length(RR_intervals)==1
        RR_intervals = [RR_intervals RR_intervals];
    end
    if length(Burak_RR_intervals)==1
        Burak_RR_intervals = [Burak_RR_intervals Burak_RR_intervals];
    end
    
    % 1 R-value = empty RR-interval vector (causes NaNs and empty values later on...)
    if isempty(RR_intervals)
        RR_intervals = [0 0];
    end
    if isempty(Burak_RR_intervals)
        Burak_RR_intervals = [0 0];
    end
    
    %% Interpolate the RR-Intervals (using Burak's RR-Interval Code)
    % Normalize all sorted RR-interval lists to 40 values via interpolation or decimation.
    interpolated_RR_intervals = interp1(1:length(Burak_RR_intervals),sort(Burak_RR_intervals),linspace(1,length(Burak_RR_intervals),interp_length));
    
    % Cap RR-intervals at 2 seconds (to avoid large errors due to outliers)
    % This is the value that is returned when the function ends and it is used to create a sparse
    % coding dictionary.
    interpolated_RR_intervals(interpolated_RR_intervals>2) = 2;
    
    %% Extract Features
    features = [];
    
    % Features 1-5: ECG Statistics
    m_ecg = mean(ecg);                                  % Feature 1 = mean ECG value
    sd_ecg = std(ecg);                                  % Feature 2 = std ECG value
    med_ecg = median(ecg);                              % Feature 3 = median ECG value
    skew_ecg = abs(mean(((ecg-m_ecg)/sd_ecg).^3));      % Feature 4 = skew ECG value (3rd moment)
    kurt_ecg = abs(mean(((ecg-m_ecg)/sd_ecg).^4));      % Feature 5 = kurtosis ECG value (4th moment)
    
    features = [features, m_ecg, sd_ecg, med_ecg, skew_ecg, kurt_ecg];
    
    % Features 6-10: RR-Interval Statistics (Challenge Code)
    m_RR = mean(RR_intervals);                          % Feature 6 = mean RR-interval value
    sd_RR = std(RR_intervals);                          % Feature 7 = std RR-interval value
    med_RR = median(RR_intervals);                      % Feature 8 = median RR-interval value
    min_RR = min(RR_intervals);                         % Feature 9 = min RR_interval value
    max_RR = max(RR_intervals);                         % Feature 10 = max RR_interval value
    
    features = [features, m_RR, sd_RR, med_RR, min_RR, max_RR];
    
    % Features 11-15: RR-Interval Statistics (Burak Code)
    m_RR2 = mean(Burak_RR_intervals);                   % Feature 11 = mean RR-interval value
    sd_RR2 = std(Burak_RR_intervals);                   % Feature 12 = std RR-interval value
    med_RR2 = median(Burak_RR_intervals);               % Feature 13 = median RR-interval value
    min_RR2 = min(Burak_RR_intervals);                  % Feature 14 = min RR_interval value
    max_RR2 = max(Burak_RR_intervals);                  % Feature 15 = max RR_interval value
    
    features = [features, m_RR2, sd_RR2, med_RR2, min_RR2, max_RR2];
    
    % Features 16-18: R-Peak Statistics (Challenge Code)
    m_R = mean(R_values);                               % Feature 16 = mean R-value
    sd_R = std(R_values);                               % Feature 17 = std R-value
    med_R = median(R_values);                           % Feature 18 = median R-value

    features = [features, m_R, sd_R, med_R];
    
    % Features 19-21: R-Peak Statistics (Burak Code)
    m_R2 = mean(R_values);                              % Feature 19 = mean R-value
    sd_R2 = std(R_values);                              % Feature 20 = std R-value
    med_R2 = median(R_values);                          % Feature 21 = median R-value

    features = [features, m_R2, sd_R2, med_R2];
    
    % Feature 22: Heart Rate (# R-peaks per second, determined by Challenge code)
    rate = length(R_times)/tm(end);                     % Feature 22 = number of R-peaks per second
    features = [features, rate];
    
    % Features 23-26: Irregularity Features (Challenge RR Intervals)
    try
        [AFEv, PACEv, IrrEv, OriginCount] = features_irr(RR_intervals');
    catch
        AFEv = 0;
        PACEv = 0;
        IrrEv = 0;
        OriginCount = 0;
    end
    
    features = [features, AFEv, PACEv, IrrEv, OriginCount];
    
    % Features 27-30: Irregularity Features (Burak RR Intervals)
    try
        [AFEv2, PACEv2, IrrEv2, OriginCount2] = features_irr(RR_intervals');  % Features 19-22 = AFEv, PACEv, IrrEv, OriginCount
    catch
        AFEv2 = 0;
        PACEv2 = 0;
        IrrEv2 = 0;
        OriginCount2 = 0;
    end
    
    features = [features, AFEv2, PACEv2, IrrEv2, OriginCount2];
    
    % Features 31-33: Entropy Features (Challenge RR Intervals)
    [Ap_En, Samp_En, CoS_En] = features_entropy(ecg,m_RR);
    
    features = [features, Ap_En, Samp_En, CoS_En];
    
    % Features 34-36: Entropy Features (Burak RR Intervals)
    [Ap_En2, Samp_En2, CoS_En2] = features_entropy(ecg,m_RR2);
    
    features = [features, Ap_En2, Samp_En2, CoS_En2];
    
    % Feature 37: Interpolated RR-interval 15-85% range
    rng_RR = interpolated_RR_intervals(end-5) - interpolated_RR_intervals(5); % Feature 26 = 'normalized range' of RR-intervals
    
    features = [features, rng_RR];
    
    % Feature 38: Number of noisy 6-second segments (from Burak's code)
    features = [features, numBadSegs];
    
    % Features 39-40: Mean and Std. Dev. of differences between features extracted from Challenge R-peak
    % code and Burak's R-peak code
    
    challenge_features = [features(6:10), features(16:18), features(23:26), features(31:33)];
    burak_features = [features(11:15), features(19:21), features(27:30), features(34:36)];
    
    mean_diff = mean(challenge_features - burak_features);
    std_diff = std(challenge_features - burak_features);
    
    features = [features, mean_diff, std_diff];
    
    % Features 41-74: Statistics from PQRST decomposition
    Burak_features = features_Burak_ECG(ecg,fs);
    
    features = [features, Burak_features'];
    
    %% Take care of NaN-valued features
    if sum(isnan(features)) > 0
        display(recordName); features
    end
    
    features(isnan(features)) = 0;
       
end