function feVec = features_Burak_ECG(signal, fs)
% Takes in an ECG signal and produces a feature vector.
% If R peak detection fails, it returns a vector of NaNs.
% Input: signal, ECG signal
% fs: sampling frequency
% feVec row feature vector.
% Features:
% P RMS (mean, std, median)
% R RMS (mean, std, median)
% T RMS (mean, std, median)
% PR (mean, std, median)
% RT (mean, std, median)
% QS (mean, std, median)
% P normalized amplitude (mean, std, median)
% R normalized amplitude (mean, std, median)
% T normalized amplitude (mean, std, median)
% S normalized amplitude (mean, std, median)
% RS normalized amplitude (mean, std, median)
% percent of the pos polarity r peaks.
% Iso mean (mean, std, median) (To be added)
% Iso median (mean, std, median) (To be added)
% Iso std (mean, std, median) (To be added)

persistent ECGHpFilt
if (isempty(ECGHpFilt))
    passbandRipple = 0.5;
    stopbandAtt = 50;
    % ECG band-pass filter
    ECGStopbandFreq1 = 0.5;
    ECGPassbandFreq1 = 3;
    % ECGPassbandFreq2 = 25;
    % ECGStopbandFreq2 = 30;
    ECGHpFilt = designfilt('highpassfir', 'StopbandFrequency', ...
        ECGStopbandFreq1, 'PassbandFrequency', ECGPassbandFreq1, ...
        'PassbandRipple', passbandRipple, 'StopbandAttenuation', ...
        stopbandAtt, 'SampleRate', fs);
end
feVec = zeros(34,1);
[ecgDel, sign, badSegs] = BurakECGDel(signal, fs, 6, ECGHpFilt, false);
signal = filtfilt(ECGHpFilt, signal);
if (sign ~= 0)
    rPeaks = ecgDel.rPeaks;
    pPts = ecgDel.pPoints;
    qPts = ecgDel.qPoints;
    sPts = ecgDel.sPoints;
    tPts = ecgDel.tPoints;
    mat = zeros(11,length(rPeaks));
    % each row represents a certain series.
    % for example P RMS values.
    for i = 1:length(rPeaks)
        % P related features
        if (~isnan(pPts(i,1)))
            curSig = signal(pPts(i,2):pPts(i,3));
            mat(1,i) = norm(curSig).^2;
            mat(2,i) = signal(pPts(i,1));
        else
            mat(1:2,i) = NaN;
        end
        % R related features
        if (~isnan(qPts(i)) && ~isnan(sPts(i,1)))
            curSig = signal(qPts(i):sPts(i,1));
            mat(3,i) = norm(curSig).^2;
        else
            mat(3,i) = NaN;
        end
        mat(4,i) = signal(rPeaks(i));
        % T related features
        if (~isnan(tPts(i,1)))
            curSig = signal(tPts(i,2):tPts(i,3));
            mat(5,i) = norm(curSig).^2;
            mat(6,i) = signal(tPts(i));
        else
            mat(5:6,i) = NaN;
        end
        
        % PR related features
        if (~isnan(pPts(i,1)))
            mat(7,i) = rPeaks(i)-pPts(i,1);
        else
            mat(7,i) = NaN;
        end
        % RT related features
        if (~isnan(tPts(i,1)))
            mat(8,i) = tPts(i,1)-rPeaks(i);
        else
            mat(8,i) = NaN;
        end
        % QS related features
        if (~isnan(qPts(i)) && ~isnan(sPts(i,1)))
            mat(9,i) = sPts(i,1) - qPts(i);
        else
            mat(9,i) = NaN;
        end
        % S related features
        if (~isnan(sPts(i,1)))
            mat(10,i) = signal(sPts(i,1));
        else
            mat(10,i) = NaN;
        end
        % RS related features
        if (~isnan(sPts(i,1)))
            mat(11,i) = signal(rPeaks(i)) - signal(sPts(i,1));
        else
            mat(11,i) = NaN;
        end
        
    end
    meanVec = mean(mat,2,'omitnan');
    stdVec = std(mat,0,2,'omitnan');
    medianVec = median(mat,2,'omitnan');
    feVec(1:11) = meanVec;
    feVec(12:22) = stdVec;
    feVec(23:33) = medianVec;
    feVec(34) = nnz(ecgDel.rPols == 1) / length(ecgDel.rPols);
else
    feVec(1:end) = NaN;
end
end